;------------------------------------------------------------------------------
;
; Z80 Monitor Rom
;
; This is Grant Searle's Z80 single board computer code modified to assemble and emulate on Zeus.
;
; In Zeus (>=v3.00) you should be able to hit "Assemble then Emulate" to assemble and run this.
; When you click on "Assemble then Emulate" Zeus should load the sourcecode for a BASIC interpreter
; and CP/M, assemble them all and start emulating them... this will change you over to the
; Emulator tab and Zeus will also emulate a simple serial terminal connected to a UART on the
; emulated Z80 hardware - you should see the message "Z80 SBC Boot ROM, etc" displayed in the
; terminal window.
;
; You should have a flashing cursor in the terminal window and be able to interact with the Z80
; code running in the emulator.
;
; Try pressing B then C then enter - this should start a Z80 BASIC interpreter.
;
; You can single-step through the source code by returning to this editor window, assembling
; the source code using the "Assemble" button then pressing F7... press F8 to run and F9 to stop.
;
; Zeus emulates Grant Searle's Z80 SBC hardware except not with an SIO but with a simpler UART,
; as the SIO is a complicated beast to fully emulate and there's really no need. Zeus emulates
; the SBC's CF card interface well enough to run CP/M... in order to do this you need to set up
; a file on your computer that emulates the CF card - I have provided a ZIP file of the contents
; of this containing some CP/M utilities and drives, all you need to do is unzip "CPM_DRIVE.ZIP"
; in a known location on your PC and tell Zeus where that file is - look on the emulator tab for
; the text box under "Path to a 128MB file used to hold the CP/M drives..." and put the full
; path and filename in there. For example on my machine the text box contains:
; "c:\temp\cpm_drive.bin" (without the quotes)
; DO NOT click on "Create CP/M drives" unless you want to create a blank 128MB file! If you did
; just unzip the "CPM_DRIVE.ZIP" file again.
;
; Once you have the CPM drive file sorted out you can assemble and run this source then press "X"
; in the terminal emulator, then "Y" and you should be running CP/M... note that this has loaded
; the CP/M binary off the "disk" and is not running the actual code you assembled here, but as the
; bytes are the same you can still single-step the CP/M code...
;
; If you want to change the CP/M code you need to assemble Grant's "putsys" and use it to update
; the CP/M binary in the emulated drive file, exactly as you would have to on a real board.
;
; The three "Send Hex File" buttons can be used to stream text files through the UART, which is
; handy for sending Grant's "cpm211filespkg.txt" file to the emulated Z80.
;
; Changes made to the CP/M drives are persistent as long as the file containing those drives is
; not deleted.
;
; Have fun... Simon.
;
; I've moved the original banner comment to the end, out of the way. Feel free to delete this lot
; when you've understood it.

;------------------------------------------------------------------------------
; General Equates
;------------------------------------------------------------------------------

                        zeusemulate "cpm"               ; Tell the emulator what we actually are

bEmulatedUART           equ true                        ; If set, we're using Zeus's emulated UART

                        profile=true                    ; Allow profiling if anyone's interested.

CR                      equ    $0D
LF                      equ    $0A
ESC                     equ    $1B
CTRLC                   equ    $03
cCLS                    equ    $0C

; CF registers (Zeus only supports the ones required to read/write sectors)
CF_DATA                 equ    $10
CF_FEATURES             equ    $11
CF_ERROR                equ    $11
CF_SECCOUNT             equ    $12
CF_SECTOR               equ    $13
CF_CYL_LOW              equ    $14
CF_CYL_HI               equ    $15
CF_HEAD                 equ    $16
CF_STATUS               equ    $17
CF_COMMAND              equ    $17
CF_LBA0                 equ    $13
CF_LBA1                 equ    $14
CF_LBA2                 equ    $15
CF_LBA3                 equ    $16

;CF Features
CF_8BIT                 equ    1
CF_NOCACHE              equ    $82
;CF Commands
CF_READ_SEC             equ    $20
CF_WRITE_SEC            equ    $30
CF_SET_FEAT             equ    $EF


loadAddr                equ    $D000                    ; CP/M load address
numSecs                 equ    24                       ; Number of 512 sectors to be loaded


;BASIC cold and warm entry points
BASCLD                  equ    $2000
BASWRM                  equ    $2003

                        org    $4000			; Why not much lower? [was set to $4100 by accident]

        if enabled bEmulatedUART
uartA_Status            equ 0
uartA_Data              equ 1
uartA_Control           equ 2
uartB_Status            equ 4
uartB_Data              equ 5
uartB_Control           equ 6
        else
SER_BUFSIZE             equ    $40
SER_FULLSIZE            equ    $30
SER_EMPTYSIZE           equ    5

RTS_HIGH                equ    $E8
RTS_LOW                 equ    $EA

SIOA_D                  equ    $00
SIOA_C                  equ    $02
SIOB_D                  equ    $01
SIOB_C                  equ    $03

serABuf                 ds     SER_BUFSIZE
serAInPtr               ds     2
serARdPtr               ds     2
serABufUsed             ds     1
serBBuf                 ds     SER_BUFSIZE
serBInPtr               ds     2
serBRdPtr               ds     2
serBBufUsed             ds     1
        endif

primaryIO               ds     1
secNo                   ds     1
bootdmaAddr             ds     2

stackSpace              ds     32
BOOTSTACK               equ    *                        ; Stack top


;------------------------------------------------------------------------------
;                         START OF MONITOR ROM
;------------------------------------------------------------------------------

                        org    $0000                    ; MONITOR ROM RESET VECTOR
MON                     equ *
;------------------------------------------------------------------------------
; Reset
;------------------------------------------------------------------------------
RST00                   di                              ;Disable INTerrupts
                        jp      BOOTINIT                ;Initialize Hardware and go
                        nop
                        nop
                        nop
                        nop
;------------------------------------------------------------------------------
; TX a character over RS232 wait for TXDONE first.
;------------------------------------------------------------------------------
RST08                   jp      bootconout
                        nop
                        nop
                        nop
                        nop
                        nop
;------------------------------------------------------------------------------
; RX a character from buffer wait until char ready.
;------------------------------------------------------------------------------
RST10                   jp      bootconin
                        nop
                        nop
                        nop
                        nop
                        nop
;------------------------------------------------------------------------------
; Check input buffer status
;------------------------------------------------------------------------------
RST18                   jp      CKINCHAR

        if enabled bEmulatedUART

serialInt               reti

; See if there's a character waiting

CKINCHAR                ld      a,(primaryIO)           ; Which IO?
                        cp      0                       ;
                        jr      nz,ckincharB            ; Try B

ckincharA               in a,(uartA_Status)             ; Is there an Rx character waiting?
                        bit 0,a                         ;
                        ret                             ; NZ = yes

ckincharB               in a,(uartB_Status)             ; Is there an Rx character waiting?
                        bit 0,a                         ;
                        ret                             ; NZ = yes

; Send a serial character to our virtual UART

bootconout              push af                         ; Save it

                        ld a,(primaryIO)                ; Which IO?
                        or a                            ;
                        jr nz bootconoutBWait           ; Goto B

                        pop af                          ;

                        ; Send a character on UART A

bootconoutA             push af                         ; Save

bootconoutAWait         in a,(uartA_Status)             ; Is the TX clear? Bit 1
                        bit 1,a                         ;
                        jr nz bootconoutAWait           ; No, loop

                        pop af                          ; Restore
                        out (uartA_Data),a              ; Send it
                        ret                             ; Done

; Send a character on UART B

bootconoutB             push af                         ; Save

bootconoutBWait         in a,(uartB_Status)             ; Is the TX clear? Bit 1
                        bit 1,a                         ;
                        jr nz bootconoutBWait           ; No, loop

                        pop af                          ; Restore
                        out (uartB_Data),a              ; Send it
                        ret                             ; Done

; Wait for a serial character from our virtual UART

bootconin               ld a,(primaryIO)                ; Which IO?
                        or a                            ;
                        jr nz bootconinB                ; Goto B

                        ; Wait for a character on UART A

bootconinA              in a,(uartA_Status)             ; Is there one ready? Bit 0
                        bit 0,a                         ;
                        jr z bootconinA                 ; No, loop

                        in a,(uartA_Data)               ; Get it
                        ret                             ; Done

; Wait for a character on UART B

bootconinB              in a,(uartB_Status)             ; Is there one?
                        bit 0,a                         ;
                        jr z bootconinB                 ; No, loop

                        in a,(uartB_Data)               ; Get it
                        ret                             ; Done

        else

;------------------------------------------------------------------------------
; SIO Vector = 0x60
;------------------------------------------------------------------------------

                        org    $0060
                        dw     serialInt

;------------------------------------------------------------------------------
; Serial interrupt handlers
; Same interrupt called if either of the inputs receives a character
; so need to check the status of each SIO input.
;------------------------------------------------------------------------------
serialInt:              push    af
                        push    hl

                        ; Check if there is a char in channel A
                        ; If not, there is a char in channel B
                        sub     a
                        out     (SIOA_C),a
                        in      a,(SIOA_C)              ; Status byte D2=TX Buff Empty, D0=RX char ready
                        rrca                            ; Rotates RX status into Carry Flag,
                        jr      nc, serialIntB

serialIntA:
                        ld      hl,(serAInPtr)
                        inc     hl
                        ld      a,l
                        cp      (serABuf+SER_BUFSIZE) & $FF
                        jr      nz, notAWrap
                        ld      hl,serABuf
notAWrap:
                        ld      (serAInPtr),hl
                        in      a,(SIOA_D)
                        ld      (hl),a

                        ld      a,(serABufUsed)
                        inc     a
                        ld      (serABufUsed),a
                        cp      SER_FULLSIZE
                        jr      c,rtsA0
                        ld      a,$05
                        out     (SIOA_C),a
                        ld      a,RTS_HIGH
                        out     (SIOA_C),a
rtsA0:
                        pop     hl
                        pop     af
                        ei
                        reti

serialIntB:
                        ld      hl,(serBInPtr)
                        inc     hl
                        ld      a,l
                        cp      (serBBuf+SER_BUFSIZE) & $FF
                        jr      nz, notBWrap
                        ld      hl,serBBuf
notBWrap:
                        ld      (serBInPtr),hl
                        in      a,(SIOB_D)
                        ld      (hl),a

                        ld      a,(serBBufUsed)
                        inc     a
                        ld      (serBBufUsed),a
                        cp      SER_FULLSIZE
                        jr      c,rtsB0
                        ld      a,$05
                        out     (SIOB_C),a
                        ld      a,RTS_HIGH
                        out     (SIOB_C),a
rtsB0:
                        pop     hl
                        pop     af
                        ei
                        reti

;------------------------------------------------------------------------------
; Console input routine
; Use the "primaryIO" flag to determine which input port to monitor.
;------------------------------------------------------------------------------
bootconin:
                        push    hl
                        ld      a,(primaryIO)
                        cp      0
                        jr      nz,bootconinB
bootconinA:

waitForCharA:
                        ld      a,(serABufUsed)
                        cp      $00
                        jr      z, waitForCharA
                        ld      hl,(serARdPtr)
                        inc     hl
                        ld      a,l
                        cp      (serABuf+SER_BUFSIZE) & $FF
                        jr      nz, notRdWrapA
                        ld      hl,serABuf
notRdWrapA:
                        di
                        ld      (serARdPtr),hl

                        ld      a,(serABufUsed)
                        dec     a
                        ld      (serABufUsed),a

                        cp      SER_EMPTYSIZE
                        jr      nc,rtsA1
                        ld      a,$05
                        out     (SIOA_C),a
                        ld      a,RTS_LOW
                        out     (SIOA_C),a
rtsA1:
                        ld      a,(hl)
                        ei

                        pop     hl

                        ret                             ; Char ready in A


bootconinB:

waitForCharB:
                        ld      a,(serBBufUsed)
                        cp      $00
                        jr      z, waitForCharB
                        ld      hl,(serBRdPtr)
                        inc     hl
                        ld      a,l
                        cp      (serBBuf+SER_BUFSIZE) & $FF
                        jr      nz, notRdWrapB
                        ld      hl,serBBuf
notRdWrapB:
                        di
                        ld      (serBRdPtr),hl

                        ld      a,(serBBufUsed)
                        dec     a
                        ld      (serBBufUsed),a

                        cp      SER_EMPTYSIZE
                        jr      nc,rtsB1
                        ld      a,$05
                        out     (SIOB_C),a
                        ld      a,RTS_LOW
                        out     (SIOB_C),a
rtsB1:
                        ld      a,(hl)
                        ei

                        pop     hl

                        ret                             ; Char ready in A

;------------------------------------------------------------------------------
; Console output routine
; Use the "primaryIO" flag to determine which output port to send a character.
;------------------------------------------------------------------------------
bootconout:             push    af                      ; Store character
                        ld      a,(primaryIO)
                        cp      0
                        jr      nz,bootconoutB1
                        jr      bootconoutA1
bootconoutA:
                        push    af

bootconoutA1:           call    CKSIOA                  ; See if SIO channel A is finished transmitting
                        jr      z,bootconoutA1          ; Loop until SIO flag signals ready
                        pop     af                      ; RETrieve character
                        out     (SIOA_D),a              ; OUTput the character
                        ret

bootconoutB:
                        push    af

bootconoutB1:           call    CKSIOB                  ; See if SIO channel B is finished transmitting
                        jr      z,bootconoutB1          ; Loop until SIO flag signals ready
                        pop     af                      ; RETrieve character
                        out     (SIOB_D),a              ; OUTput the character
                        ret

;------------------------------------------------------------------------------
; I/O status check routine
; Use the "primaryIO" flag to determine which port to check.
;------------------------------------------------------------------------------
CKSIOA                  sub     a
                        out     (SIOA_C),a
                        in      a,(SIOA_C)              ; Status byte D2=TX Buff Empty, D0=RX char ready
                        rrca                            ; Rotates RX status into Carry Flag,
                        bit     1,a                     ; Set Zero flag if still transmitting character
                        ret

CKSIOB                  sub     a
                        out     (SIOB_C),a
                        in      a,(SIOB_C)              ; Status byte D2=TX Buff Empty, D0=RX char ready
                        rrca                            ; Rotates RX status into Carry Flag,
                        bit     1,a                     ; Set Zero flag if still transmitting character
                        ret

;------------------------------------------------------------------------------
; Check if there is a character in the input buffer
; Use the "primaryIO" flag to determine which port to check.
;------------------------------------------------------------------------------
CKINCHAR                ld      a,(primaryIO)
                        cp      0
                        jr      nz,ckincharB

ckincharA:

                        ld      a,(serABufUsed)
                        cp      $0
                        ret

ckincharB:

                        ld      a,(serBBufUsed)
                        cp      $0
                        ret
        endif

;------------------------------------------------------------------------------
; Filtered Character I/O
;------------------------------------------------------------------------------

RDCHR                   rst     $10
                        cp      LF
                        jr      z,RDCHR                 ; Ignore LF
                        cp      ESC
                        jr      nz,RDCHR1
                        ld      a,CTRLC                 ; Change ESC to CTRL-C
RDCHR1                  ret

WRCHR                   cp      CR
                        jr      z,WRCRLF                ; When CR, write CRLF
                        cp      cCLS
                        jr      z,WR                    ; Allow write of "CLS"
                        cp      ' '                     ; Don't write out any other control codes
                        jr      c,NOWR                  ; ie. < space
WR                      rst     $08
NOWR                    ret

WRCRLF                  ld      a,CR
                        rst     $08
                        ld      a,LF
                        rst     $08
                        ld      a,CR
                        ret


;------------------------------------------------------------------------------
; Initialise hardware and start main loop
;------------------------------------------------------------------------------
BOOTINIT                ld   sp,BOOTSTACK               ; Set the Stack Pointer

;       Initialise SIO

        if enabled bEmulatedUART

; We don't need to use interrupts or buffer with the emulated UART. The PC has *large* buffers

                        ld      a,$00                   ; Zeus uses channel 0
                        ld      (primaryIO),a

        else
                        ld      hl,serABuf
                        ld      (serAInPtr),hl
                        ld      (serARdPtr),hl

                        ld      hl,serBBuf
                        ld      (serBInPtr),hl
                        ld      (serBRdPtr),hl

                        xor     a                       ;0 to accumulator
                        ld      (serABufUsed),a
                        ld      (serBBufUsed),a

                        ld      a,$00
                        out     (SIOA_C),a
                        ld      a,$18
                        out     (SIOA_C),a

                        ld      a,$04
                        out     (SIOA_C),a
                        ld      a,$C4
                        out     (SIOA_C),a

                        ld      a,$01
                        out     (SIOA_C),a
                        ld      a,$18
                        out     (SIOA_C),a

                        ld      a,$03
                        out     (SIOA_C),a
                        ld      a,$E1
                        out     (SIOA_C),a

                        ld      a,$05
                        out     (SIOA_C),a
                        ld      a,RTS_LOW
                        out     (SIOA_C),a

                        ld      a,$00
                        out     (SIOB_C),a
                        ld      a,$18
                        out     (SIOB_C),a

                        ld      a,$04
                        out     (SIOB_C),a
                        ld      a,$C4
                        out     (SIOB_C),a

                        ld      a,$01
                        out     (SIOB_C),a
                        ld      a,$18
                        out     (SIOB_C),a

                        ld      a,$02
                        out     (SIOB_C),a
                        ld      a,$60                   ; INTERRUPT VECTOR ADDRESS
                        out     (SIOB_C),a

                        ld      a,$03
                        out     (SIOB_C),a
                        ld      a,$E1
                        out     (SIOB_C),a

                        ld      a,$05
                        out     (SIOB_C),a
                        ld      a,RTS_LOW
                        out     (SIOB_C),a

                        ; Interrupt vector in page 0
                        ld      a,$00
                        ld      i,a

                        im      2
                        ei

                        ; Display the "Press space to start" message on both consoles
                        ld      a,$00
                        ld      (primaryIO),a
                        ld      hl,INITTXT
                        call    BOOTPRINT
                        ld      a,$01
                        ld      (primaryIO),a
                        ld      hl,INITTXT
                        call    BOOTPRINT

                        ; Wait until space is in one of the buffers to determine the active console

waitForSpace:

                        call ckincharA
                        jr      z,notInA
                        ld      a,$00
                        ld      (primaryIO),a
                        call    bootconin
                        cp      ' '
                        jp      nz, waitForSpace
                        jr      spacePressed

notInA:
                        call ckincharB
                        jr      z,waitForSpace
                        ld      a,$01
                        ld      (primaryIO),a
                        call    bootconin
                        cp      ' '
                        jp      nz, waitForSpace
                        jr      spacePressed

spacePressed:

                        ; Clear message on both consoles
                        ld      a,$0C
                        call    bootconoutA
                        call    bootconoutB

        endif
                        ; primaryIO is now set to the channel where SPACE was pressed


                        call TXCRLF                     ; TXCRLF
                        ld   hl,szBOOTSIGNON            ; Print SIGNON message
                        call BOOTPRINT

;------------------------------------------------------------------------------
; Monitor command loop
;------------------------------------------------------------------------------
MAIN                    ld   hl,MAIN                    ; Save entry point for Monitor
                        push hl                         ; This is the return address
MAIN0                   call TXCRLF                     ; Entry point for Monitor, Normal
                        ld   a,'>'                      ; Get a ">"
                        rst $08                         ; print it

MAIN1                   call RDCHR                      ; Get a character from the input port
                        cp   ' '                        ; <spc> or less?
                        jr   c,MAIN1                    ; Go back

                        cp   ':'                        ; ":"?
                        jp   z,LOAD                     ; First character of a HEX load

                        call WRCHR                      ; Print char on console

                        cp   '?'
                        jp   z,HELP

                        and  $5F                        ; Make character uppercase

                        cp   'R'
                        jp   z,RST00

                        cp   'B'
                        jp   z,BASIC

                        cp   'G'
                        jp   z,BOOTGOTO

                        cp   'X'
                        jp   z,CPMLOAD

                        ld   a,'?'                      ; Get a "?"
                        rst $08                         ; Print it
                        jr   MAIN0

;------------------------------------------------------------------------------
; Print string of characters to Serial A until byte=$00, WITH CR, LF
;------------------------------------------------------------------------------
BOOTPRINT               ld   a,(hl)                     ; Get character
                        or   a                          ; Is it $00 ?
                        ret  z                          ; Then RETurn on terminator
                        rst  $08                        ; Print it
                        inc  hl                         ; Next Character
                        jr   BOOTPRINT                  ; Continue until $00


TXCRLF                  ld   a,$0D                      ;
                        rst  $08                        ; Print character
                        ld   a,$0A                      ;
                        rst  $08                        ; Print character
                        ret

;------------------------------------------------------------------------------
; Get a character from the console, must be $20-$7F to be valid (no control characters)
; <Ctrl-c> and <SPACE> breaks with the Zero Flag set
;------------------------------------------------------------------------------
BOOTGETCHR              call RDCHR                      ; RX a Character
                        cp   $03                        ; <ctrl-c> User break?
                        ret  z
                        cp   $20                        ; <space> or better?
                        jr   c,BOOTGETCHR               ; Do it again until we get something usable
                        ret
;------------------------------------------------------------------------------
; Gets two ASCII characters from the console (assuming them to be HEX 0-9 A-F)
; Moves them into B and C, converts them into a byte value in A and updates a
; Checksum value in E
;------------------------------------------------------------------------------
GET2                    call BOOTGETCHR                 ; Get us a valid character to work with
                        ld   b,a                        ; Load it in B
                        call BOOTGETCHR                 ; Get us another character
                        ld   c,a                        ; load it in C
                        call BCTOA                      ; Convert ASCII to byte
                        ld   c,a                        ; Build the checksum
                        ld   a,e
                        sub  c                          ; The checksum should always equal zero when checked
                        ld   e,a                        ; Save the checksum back where it came from
                        ld   a,c                        ; Retrieve the byte and go back
                        ret
;------------------------------------------------------------------------------
; Gets four Hex characters from the console, converts them to values in HL
;------------------------------------------------------------------------------
GETHL                   ld   hl,$0000                   ; Gets xxxx but sets Carry Flag on any Terminator
                        call ECHO                       ; RX a Character
                        cp   $0D                        ; <CR>?
                        jr   nz,GETX2                   ; other key
SETCY                   scf                             ; Set Carry Flag
                        ret                             ; and Return to main program
;------------------------------------------------------------------------------
; This routine converts last four hex characters (0-9 A-F) user types into a value in HL
; Rotates the old out and replaces with the new until the user hits a terminating character
;------------------------------------------------------------------------------
GETX                    ld   hl,$0000                   ; CLEAR HL
GETX1                   call ECHO                       ; RX a character from the console
                        cp   $0D                        ; <CR>
                        ret  z                          ; quit
                        cp   $2C                        ; <,> can be used to safely quit for multiple entries
                        ret  z                          ; (Like filling both DE and HL from the user)
GETX2                   cp   $03                        ; Likewise, a <ctrl-C> will terminate clean, too, but
                        jr   z,SETCY                    ; It also sets the Carry Flag for testing later.
                        add  hl,hl                      ; Otherwise, rotate the previous low nibble to high
                        add  hl,hl                      ; rather slowly
                        add  hl,hl                      ; until we get to the top
                        add  hl,hl                      ; and then we can continue on.
                        sub  $30                        ; Convert ASCII to byte value
                        cp   $0A                        ; Are we in the 0-9 range?
                        jr   c,GETX3                    ; Then we just need to sub $30, but if it is A-F
                        sub  $07                        ; We need to take off 7 more to get the value down to
GETX3                   and  $0F                        ; to the right hex value
                        add  a,l                        ; Add the high nibble to the low
                        ld   l,a                        ; Move the byte back to A
                        jr   GETX1                      ; and go back for next character until he terminates
;------------------------------------------------------------------------------
; Convert ASCII characters in B C registers to a byte value in A
;------------------------------------------------------------------------------
BCTOA                   ld   a,b                        ; Move the hi order byte to A
                        sub  $30                        ; Take it down from Ascii
                        cp   $0A                        ; Are we in the 0-9 range here?
                        jr   c,BCTOA1                   ; If so, get the next nybble
                        sub  $07                        ; But if A-F, take it down some more
BCTOA1                  rlca                            ; Rotate the nybble from low to high
                        rlca                            ; One bit at a time
                        rlca                            ; Until we
                        rlca                            ; Get there with it
                        ld   b,a                        ; Save the converted high nybble
                        ld   a,c                        ; Now get the low order byte
                        sub  $30                        ; Convert it down from Ascii
                        cp   $0A                        ; 0-9 at this point?
                        jr   c,BCTOA2                   ; Good enough then, but
                        sub  $07                        ; Take off 7 more if it's A-F
BCTOA2                  add  a,b                        ; Add in the high order nybble
                        ret

;------------------------------------------------------------------------------
; Get a character and echo it back to the user
;------------------------------------------------------------------------------
ECHO                    call    RDCHR
                        call    WRCHR
                        ret

;------------------------------------------------------------------------------
; BOOTGOTO command
;------------------------------------------------------------------------------
BOOTGOTO                call GETHL                      ; ENTRY POINT FOR <G>oto addr. Get XXXX from user.
                        ret  c                          ; Return if invalid
                        push hl
                        ret                             ; Jump to HL address value

;------------------------------------------------------------------------------
; LOAD Intel Hex format file from the console.
; [Intel Hex Format is:
; 1) Colon (Frame 0)
; 2) Record Length Field (Frames 1 and 2)
; 3) Load Address Field (Frames 3,4,5,6)
; 4) Record Type Field (Frames 7 and 8)
; 5) Data Field (Frames 9 to 9+2*(Record Length)-1
; 6) Checksum Field - Sum of all byte values from Record Length to and
;   including Checksum Field = 0 ]
;------------------------------------------------------------------------------
LOAD                    ld   e,0                        ; First two Characters is the Record Length Field
                        call GET2                       ; Get us two characters into BC, convert it to a byte <A>
                        ld   d,a                        ; Load Record Length count into D
                        call GET2                       ; Get next two characters, Memory Load Address <H>
                        ld   h,a                        ; put value in H register.
                        call GET2                       ; Get next two characters, Memory Load Address <L>
                        ld   l,a                        ; put value in L register.
                        call GET2                       ; Get next two characters, Record Field Type
                        cp   $01                        ; Record Field Type 00 is Data, 01 is End of File
                        jr   nz,LOAD2                   ; Must be the end of that file
                        call GET2                       ; Get next two characters, assemble into byte
                        ld   a,e                        ; Recall the Checksum byte
                        and  a                          ; Is it Zero?
                        jr   z,LOAD00                   ; Print footer reached message
                        jr   LOADERR                    ; Checksums don't add up, Error out

LOAD2                   ld   a,d                        ; Retrieve line character counter
                        and  a                          ; Are we done with this line?
                        jr   z,LOAD3                    ; Get two more ascii characters, build a byte and checksum
                        call GET2                       ; Get next two chars, convert to byte in A, checksum it
                        ld   (hl),a                     ; Move converted byte in A to memory location
                        inc  hl                         ; Increment pointer to next memory location
                        ld   a,'.'                      ; Print out a "." for every byte loaded
                        rst  $08                        ;
                        dec  d                          ; Decrement line character counter
                        jr   LOAD2                      ; and keep loading into memory until line is complete

LOAD3                   call GET2                       ; Get two chars, build byte and checksum
                        ld   a,e                        ; Check the checksum value
                        and  a                          ; Is it zero?
                        ret  z

LOADERR                 ld   hl,CKSUMERR                ; Get "Checksum Error" message
                        call BOOTPRINT                  ; Print Message from (HL) and terminate the load
                        ret

LOAD00                  ld   hl,LDETXT                  ; Print load complete message
                        call BOOTPRINT
                        ret

;------------------------------------------------------------------------------
; Start BASIC command
;------------------------------------------------------------------------------
BASIC                   ld hl,szBASTXT
                        call BOOTPRINT
                        call BOOTGETCHR
                        ret z                           ; Cancel if CTRL-C
                        and  $5F                        ; uppercase
                        cp 'C'
                        jp  z,BASCLD
                        cp 'W'
                        jp  z,BASWRM
                        ret

;------------------------------------------------------------------------------
; Display Help command
;------------------------------------------------------------------------------
HELP                    ld   hl,HLPTXT                  ; Print Help message
                        call BOOTPRINT
                        ret

;------------------------------------------------------------------------------
; CP/M load command
;------------------------------------------------------------------------------
CPMLOAD                 ld hl,CPMTXT
                        call BOOTPRINT
                        call BOOTGETCHR
                        ret z                           ; Cancel if CTRL-C
                        and  $5F                        ; uppercase
                        cp 'Y'
                        jp  z,CPMLOAD2
                        ret

CPMTXT                  db   $0D,$0A
                        db   "Boot CP/M?"
                        db   $00

CPMTXT2                 db   $0D,$0A
                        db   "Loading CP/M..."
                        db   $0D,$0A,$00

CPMLOAD2                ld hl,CPMTXT2
                        call BOOTPRINT

                        call    bootcfWait
                        ld      a,CF_8BIT               ; Set IDE to be 8bit
                        out     (CF_FEATURES),a
                        ld      a,CF_SET_FEAT
                        out     (CF_COMMAND),a


                        call    bootcfWait
                        ld      a,CF_NOCACHE            ; No write cache
                        out     (CF_FEATURES),a
                        ld      a,CF_SET_FEAT
                        out     (CF_COMMAND),a

                        ld      b,numSecs

                        ld      a,0
                        ld      (secNo),a
                        ld      hl,loadAddr
                        ld      (bootdmaAddr),hl
processSectors:

                        call    bootcfWait

                        ld      a,(secNo)
                        out     (CF_LBA0),a
                        ld      a,0
                        out     (CF_LBA1),a
                        out     (CF_LBA2),a
                        ld      a,$E0
                        out     (CF_LBA3),a
                        ld      a,1
                        out     (CF_SECCOUNT),a

                        call    bootread

                        ld      de,$0200
                        ld      hl,(bootdmaAddr)
                        add     hl,de
                        ld      (bootdmaAddr),hl
                        ld      a,(secNo)
                        inc     a
                        ld      (secNo),a

                        djnz    processSectors

; Start CP/M using entry at top of BIOS
; The current active console stream ID is pushed onto the stack
; to allow the CBIOS to pick it up
; 0 = SIO A, 1 = SIO B

                        ld      a,(primaryIO)
                        push    af
                        ld      hl,($FFFE)
                        jp      (hl)


;------------------------------------------------------------------------------

; Read physical sector from host

bootread:
                        push    af
                        push    bc
                        push    hl

                        call    bootcfWait

                        ld      a,CF_READ_SEC
                        out     (CF_COMMAND),a

                        call    bootcfWait

                        ld      c,4
                        ld      hl,(bootdmaAddr)
bootrd4secs:
                        ld      b,128
bootrdByte:
                        nop
                        nop
                        in      a,(CF_DATA)
                        ld      (hl),a
                        inc     hl
                        dec     b
                        jr      nz, bootrdByte
                        dec     c
                        jr      nz,bootrd4secs

                        pop     hl
                        pop     bc
                        pop     af

                        ret


; Wait for disk to be ready (busy=0,ready=1)
bootcfWait:
                        push    af
bootcfWait1:
                        in      a,(CF_STATUS)
                        and     $80
                        cp      $80
                        jr      z,bootcfWait1
                        pop     af
                        ret

;------------------------------------------------------------------------------

szBOOTSIGNON            db   $0C,"Z80 SBC Boot ROM 1.1"
                        db   " by G. Searle"
                        db   $0D,$0A
                        db   "(modified slightly to run under Zeus emulation)"
                        db   $0D,$0A
;                        db   "(You might need to click on this text to give the terminal focus)"
;                        db   $0D,$0A
;                        db   $0D,$0A

                        db   "Type ? for options"
                        db   $0D,$0A,$00

szBASTXT                db   $0D,$0A
                        db   "Cold or Warm ?"
                        db   $0D,$0A,$00

CKSUMERR                db   "Checksum error"
                        db   $0D,$0A,$00

INITTXT                 db   $0C
                        db   "Press [SPACE] to activate console"
                        db   $0D,$0A, $00

LDETXT                  db   "Load complete."
                        db   $0D,$0A, $00


HLPTXT                  db   $0D,$0A
                        db   "R           - Reset"
                        db   $0D,$0A
                        db   "BC or BW    - ROM BASIC Cold or Warm"
                        db   $0D,$0A
                        db   "X           - Boot CP/M (load $D000-$FFFF from disk)"
                        db   $0D,$0A
                        db   ":nnnnnn...  - Load Intel-Hex file record"
                        db   $0D,$0A
                        db   "G xxxx      - Goto code at xxxx"
                        db   $0D,$0A
                        db   $00

;------------------------------------------------------------------------------

;FINIS                   ds $3FFF-*,$FF

; Now, import the BASIC interpreter binary (assemble BASIC.ASM to produce this)

                        include "basic.asm"
;                        import_bin "basic.bin",$2000


; Load CP/M into memory while we're at it

                        include "cpm22.asm"
                        include "cbios.asm"

; This can be used to compare the code generated by these sources with the original

        if false
                        import_comparison "boot.bin",$0000
                        import_comparison "basic.bin",$2000
                        import_comparison "cpm22.bin",$D000
                        import_comparison "cbios64.bin",$E600
        endif


;==================================================================================
;
; Contents of this file are copyright Grant Searle
; HEX routines from Joel Owens.
;
; Changes to Zeus's syntax/Emulated serial port S.Brattel.
;
; You have permission to use this for NON COMMERCIAL USE ONLY
; If you wish to use it elsewhere, please include an acknowledgement to myself.
;
; http://searle.hostei.com/grant/index.html
;
; eMail: home.micros01@btinternet.com
;
; If the above don't work, please perform an Internet search to see if I have
; updated the web page hosting service.
;
;==================================================================================


